@@ -6,8 +6,7 @@ irmen@razorvine.net - http://www.razorvine.net/python/Pyro
 """
 
 import sys
-import Pyro4.naming
-import Pyro4.errors
+from Pyro4 import naming, errors
 
 if sys.version_info<(3, 0):
     input=raw_input
@@ -84,8 +83,8 @@ def main(args):
     if options.verbose:
         print("Locating name server...")
     try:
-        nameserver=Pyro4.naming.locateNS(options.host, options.port)
-    except Pyro4.errors.PyroError:
+        nameserver=naming.locateNS(options.host, options.port)
+    except errors.PyroError:
         x=sys.exc_info()[1]
         print("Failed to locate the name server: %s" % x)
         return
