@@ -8,20 +8,11 @@ irmen@razorvine.net - http://www.razorvine.net/python/Pyro
 from __future__ import with_statement
 import unittest
 import copy
-import sys
 import Pyro4.core
-import Pyro4.config
 import Pyro4.errors
+import Pyro4.constants
+from testsupport import *
 
-if sys.version_info>=(3,0):
-    unicode=str
-    unichr=chr
-if sys.version_info<(3,0):
-    def tobytes(string, encoding=None):
-        return string
-else:
-    def tobytes(string, encoding="iso-8859-1"):
-        return bytes(string,encoding)
 
 class Thing(object):
     def __init__(self, arg):
@@ -32,7 +23,7 @@ class Thing(object):
 
 class CoreTests(unittest.TestCase):
     def setUp(self):
-        Pyro4.config.HMAC_KEY="testsuite"
+        Pyro4.config.HMAC_KEY=tobytes("testsuite")
     def tearDown(self):
         Pyro4.config.HMAC_KEY=None
 
@@ -44,6 +35,13 @@ class CoreTests(unittest.TestCase):
         self.assertTrue("COMPRESSION" in config)
         self.assertEqual(Pyro4.config.COMPRESSION, config["COMPRESSION"])
 
+    def testConfigValid(self):
+        try:
+            Pyro4.config.XYZ_FOOBAR=True  # don't want to allow weird config names
+            self.fail("expected exception for weird config item")
+        except AttributeError:
+            pass
+
     def testUriStrAndRepr(self):
         uri="PYRONAME:some_obj_name"
         p=Pyro4.core.URI(uri)
@@ -209,22 +207,21 @@ class CoreTests(unittest.TestCase):
     def testMsgFactory(self):
         import hashlib, hmac
         def pyrohmac(data):
-            hmac_key = tobytes(Pyro4.config.HMAC_KEY)
             data=tobytes(data)
-            return hmac.new(hmac_key, data, digestmod=hashlib.sha1).digest()
+            return hmac.new(Pyro4.config.HMAC_KEY, data, digestmod=hashlib.sha1).digest()
         MF=Pyro4.core.MessageFactory
         MF.createMessage(99, None, 0,0) # doesn't check msg type here
         self.assertRaises(Pyro4.errors.ProtocolError, MF.parseMessageHeader, "FOOBAR")
         hdr=MF.createMessage(MF.MSG_CONNECT, tobytes("hello"),0,0)[:-5]
         msgType,flags,seq,dataLen,datahmac=MF.parseMessageHeader(hdr)
         self.assertEqual(MF.MSG_CONNECT, msgType)
-        self.assertEqual(0, flags)
+        self.assertEqual(MF.FLAGS_HMAC, flags)
         self.assertEqual(5, dataLen)
         self.assertEqual(pyrohmac("hello"), datahmac)
         hdr=MF.createMessage(MF.MSG_RESULT, None,0,0)
         msgType,flags,seq,dataLen,datahmac=MF.parseMessageHeader(hdr)
         self.assertEqual(MF.MSG_RESULT, msgType)
-        self.assertEqual(0, flags)
+        self.assertEqual(MF.FLAGS_HMAC, flags)
         self.assertEqual(0, dataLen)
         hdr=MF.createMessage(MF.MSG_RESULT, tobytes("hello"), 42, 0)[:-5]
         msgType,flags,seq,dataLen,datahmac=MF.parseMessageHeader(hdr)
@@ -245,7 +242,7 @@ class CoreTests(unittest.TestCase):
 
     def testMsgFactoryProtocolVersion(self):
         version=Pyro4.constants.PROTOCOL_VERSION
-        Pyro4.constants.PROTOCOL_VERSION=0;     # fake invalid protocol version number
+        Pyro4.constants.PROTOCOL_VERSION=0     # fake invalid protocol version number
         msg=Pyro4.core.MessageFactory.createMessage(Pyro4.core.MessageFactory.MSG_RESULT, tobytes("result"), 0, 1)
         try:
             Pyro4.core.MessageFactory.parseMessageHeader(msg)
@@ -295,6 +292,7 @@ class CoreTests(unittest.TestCase):
                 self.closeCalled=True
 
         connMock=ConnectionMock()
+        # first without a 'with' statement
         p=Pyro4.core.Proxy("PYRO:9999@localhost:15555")
         p._pyroConnection=connMock
         self.assertFalse(connMock.closeCalled)
@@ -317,7 +315,6 @@ class CoreTests(unittest.TestCase):
             pass
         self.assertTrue(p._pyroConnection is None)
         self.assertTrue(connMock.closeCalled)
-        connMock=ConnectionMock()
         p=Pyro4.core.Proxy("PYRO:9999@localhost:15555")
         with p:
             self.assertTrue(p._pyroUri is not None)
@@ -340,6 +337,15 @@ class CoreTests(unittest.TestCase):
         a2=a.nestedattribute
         self.assertTrue(isinstance(a2, Pyro4.core._RemoteMethod), "nested attribute should just be another RemoteMethod")
 
+    def testNoConnect(self):
+        wrongUri=Pyro4.core.URI("PYRO:foobar@localhost:59999")
+        with Pyro4.core.Proxy(wrongUri) as p:
+            try:
+                p.ping()
+                self.fail("CommunicationError expected")
+            except Pyro4.errors.CommunicationError:
+                pass
+
     def testTimeoutGetSet(self):
         class ConnectionMock(object):
             def __init__(self):
