@@ -11,18 +11,15 @@ import sys, select, os
 import Pyro4.core
 import Pyro4.naming
 import Pyro4.nsc
+import Pyro4.constants
+import Pyro4.socketutil
 from Pyro4.errors import NamingError,PyroError
+from testsupport import *
 
-if sys.version_info>=(3,0):
-    from io import StringIO
-    unicode=str
-    unichr=chr
-else:
-    from StringIO import StringIO
 
 class OfflineNameServerTests(unittest.TestCase):
     def setUp(self):
-        Pyro4.config.HMAC_KEY="testsuite"
+        Pyro4.config.HMAC_KEY=tobytes("testsuite")
     def tearDown(self):
         Pyro4.config.HMAC_KEY=None
     def testRegister(self):
@@ -109,7 +106,7 @@ class OfflineNameServerTests(unittest.TestCase):
     def testRefuseDotted(self):
         try:
             Pyro4.config.DOTTEDNAMES=True
-            ns=Pyro4.naming.NameServerDaemon(port=0)
+            _=Pyro4.naming.NameServerDaemon(port=0)
             self.fail("should refuse to create name server")
         except PyroError:
             pass
@@ -155,7 +152,7 @@ class OfflineNameServerTests(unittest.TestCase):
         self.assertTrue(isinstance(bc2, Pyro4.naming.BroadcastServer))
         sock=bc2.sock
         self.assertTrue(hasattr(sock,"fileno"))
-        func=bc2.processRequest
+        _=bc2.processRequest
         ns1.close()
         ns2.close()
         bc2.close()
@@ -170,15 +167,15 @@ class OfflineNameServerTests(unittest.TestCase):
                 # jython requires nonblocking sockets for poll
                 ns1.sock.setblocking(False)
                 bc1.sock.setblocking(False)
-            for s in ns1.sockets():
-                p.register(s, select.POLLIN)  #@UndefinedVariable (pydev)
-            p.register(bc1.fileno(), select.POLLIN)  #@UndefinedVariable (pydev)
+            for s in ns1.sockets:
+                p.register(s, select.POLLIN)
+            p.register(bc1.fileno(), select.POLLIN)
             p.poll(100)
             if hasattr(p,"close"):
                 p.close()
         else:
             rs=[bc1]
-            rs.extend(ns1.sockets())
+            rs.extend(ns1.sockets)
             _,_,_=select.select(rs,[],[],0.1)
         ns1.close()
         bc1.close()
