@@ -6,21 +6,15 @@ irmen@razorvine.net - http://www.razorvine.net/python/Pyro
 """
 
 import unittest
-import socket, os, sys
+import socket, os
 import Pyro4.socketutil as SU
-import Pyro4.config
 from Pyro4 import threadutil
-from Pyro4.socketserver.selectserver import SocketServer_Select
+from Pyro4.socketserver.multiplexserver import SocketServer_Select, SocketServer_Poll
 from Pyro4.socketserver.threadpoolserver import SocketServer_Threadpool
+import Pyro4
+from testsupport import *
 
-if sys.version_info<(3,0):
-    def tobytes(string, encoding=None):
-        return string
-else:
-    def tobytes(string, encoding="iso-8859-1"):
-        return bytes(string,encoding)
 
-        
 class TestSocketutil(unittest.TestCase):
     def setUp(self):
         Pyro4.config.POLLTIMEOUT=0.1
@@ -160,12 +154,12 @@ class ServerCallback(object):
 
 class TestSocketServer(unittest.TestCase):
     def testServer_thread(self):
-        callback=ServerCallback()
+        daemon=ServerCallback()
         port=SU.findUnusedPort()
-        serv=SocketServer_Threadpool(callback,"localhost",port)
+        serv=SocketServer_Threadpool()
+        serv.init(daemon,"localhost",port)
         self.assertEqual("localhost:"+str(port), serv.locationStr)
         self.assertTrue(serv.sock is not None)
-        self.assertTrue(serv.fileno() > 0)
         conn=SU.SocketConnection(serv.sock, "ID12345")
         self.assertEqual("ID12345",conn.objectId)
         self.assertTrue(conn.sock is not None)
@@ -176,16 +170,12 @@ class TestSocketServer(unittest.TestCase):
         serv.close()
         self.assertTrue(serv.sock is None)
     def testServer_select(self):
-        callback=ServerCallback()
+        daemon=ServerCallback()
         port=SU.findUnusedPort()
-        if os.name=="java":
-            # select-based server is not available in Jython
-            self.assertRaises(NotImplementedError, SocketServer_Select, callback, "localhost", port)
-            return
-        serv=SocketServer_Select(callback,"localhost",port)
+        serv=SocketServer_Select()
+        serv.init(daemon,"localhost",port)
         self.assertEqual("localhost:"+str(port), serv.locationStr)
         self.assertTrue(serv.sock is not None)
-        self.assertTrue(serv.fileno() > 0)
         conn=SU.SocketConnection(serv.sock, "ID12345")
         self.assertEqual("ID12345",conn.objectId)
         self.assertTrue(conn.sock is not None)
@@ -195,7 +185,23 @@ class TestSocketServer(unittest.TestCase):
         serv.close()
         serv.close()
         self.assertTrue(serv.sock is None)
-        
+    def testServer_poll(self):
+        daemon=ServerCallback()
+        port=SU.findUnusedPort()
+        serv=SocketServer_Poll()
+        serv.init(daemon,"localhost",port)
+        self.assertEqual("localhost:"+str(port), serv.locationStr)
+        self.assertTrue(serv.sock is not None)
+        conn=SU.SocketConnection(serv.sock, "ID12345")
+        self.assertEqual("ID12345",conn.objectId)
+        self.assertTrue(conn.sock is not None)
+        conn.close()
+        conn.close()
+        self.assertFalse(conn.sock is None, "connections keep their socket object even if it's closed")
+        serv.close()
+        serv.close()
+        self.assertTrue(serv.sock is None)
+
 
 if __name__ == "__main__":
     #import sys;sys.argv = ['', 'Test.testName']
